#!/usr/bin/env python
# encoding: utf-8

'''
@author: Tsuyoshi Hombashi
'''

from __future__ import with_statement
import sys

import thutils
import tcconfig


def parse_option():
    parser = thutils.option.ArgumentParserObject()
    parser.make(version="0.2.0")

    group = parser.add_argument_group("Traffic Control")
    group.add_argument(
        "--device", required=True,
        help="network device name")
    group.add_argument(
        "--rate",
        help="network bandwidth to apply the limit [K|M|G bps]")
    group.add_argument(
        "--delay", type=float, default=0,
        help="round trip network delay [ms] (default=%(default)s)")
    group.add_argument(
        "--loss", type=float, default=0,
        help="round trip packet loss rate [%%] (default=%(default)s)")
    group.add_argument(
        "--network",
        help="destination network of traffic control")
    group.add_argument(
        "--port", type=int,
        help="destination port of traffic control")
    group.add_argument(
        "--overwrite", action="store_true", default=False,
        help="overwrite existing setting")

    return parser.parse_args()


@thutils.main.Main
def main():
    options = parse_option()

    thutils.initialize_library(__file__, options)
    thutils.common.verify_install_command(["tc"])

    subproc_wrapper = thutils.subprocwrapper.SubprocessWrapper()
    tc = tcconfig.TrafficControl(subproc_wrapper, options.device)
    tc.rate = options.rate
    tc.delay_ms = options.delay
    tc.loss_percent = options.loss
    tc.network = options.network
    tc.port = options.port

    import logging
    subproc_wrapper.command_log_level = logging.DEBUG

    tc.validate()

    if options.overwrite:
        tc.delete_tc()

    tc.set_tc()
    #tc.set_tc(options.rate, options.delay, options.loss, options.network)

    return 0


if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python
# encoding: utf-8

'''
@author: Tsuyoshi Hombashi
'''

from __future__ import with_statement
import sys

import thutils
import tcconfig


def parse_option():
    parser = thutils.option.ArgumentParserObject()
    parser.make(version="0.2.0")

    group = parser.add_argument_group("Traffic Control")
    group.add_argument(
        "--device", required=True,
        help="network device name")

    return parser.parse_args()


@thutils.main.Main
def main():
    options = parse_option()

    thutils.initialize_library(__file__, options)

    thutils.common.verify_install_command(["tc"])

    subproc_wrapper = thutils.subprocwrapper.SubprocessWrapper()
    tc = tcconfig.TrafficControl(subproc_wrapper, options.device)

    return tc.delete_tc()


if __name__ == '__main__':
    sys.exit(main())



# encoding: utf-8

'''
@author: Tsuyoshi Hombashi
'''

import itertools

import pytest
import thutils
import tcconfig


@pytest.fixture
def subproc_wrapper():
    return thutils.subprocwrapper.SubprocessWrapper()


DEVICE = "eth0"


class NormalTestValue:
    RATE_LIST = [
        "",
        "--rate 100K",
        "--rate 0.5M",
        "--rate 0.1G",
    ]
    DELAY_LIST = [
        "",
        "--delay 0",
        "--delay 10",
        "--delay 100",
    ]
    LOSS_LIST = [
        "",
        "--loss 0",
        "--loss 0.1",
        "--loss 99",
    ]
    NETWORK_LIST = [
        "",
        "--network 192.168.0.10",
        "--network 192.168.0.10/24",
    ]
    PORT_LIST = [
        "",
        "--port 80",
    ]
    OVERWRITE_LIST = [
        "",
        "--overwrite",
    ]


class Test_tcconfig:

    @pytest.mark.parametrize(["rate", "delay", "loss", "overwrite"], [
        opt_list
        for opt_list in itertools.product(
            NormalTestValue.RATE_LIST,
            NormalTestValue.DELAY_LIST,
            NormalTestValue.LOSS_LIST,
            NormalTestValue.OVERWRITE_LIST)
    ])
    def test_smoke(self, subproc_wrapper, rate, delay, loss, overwrite):
        option_list = [rate, delay, loss, overwrite]
        command = "tcset --device %s " % (DEVICE) + " ".join(option_list)
        assert subproc_wrapper.run(command) == 0

        command = "tcdel --device %s" % (DEVICE)
        assert subproc_wrapper.run(command) == 0


@pytest.fixture
def tc_obj():
    subproc_wrapper = thutils.subprocwrapper.SubprocessWrapper()
    return tcconfig.TrafficControl(subproc_wrapper, "eth0")


@pytest.mark.parametrize(["value"], [
    [None],
    [0],
    [10000],
])
def test_TrafficControl_validate_network_delay_normal(tc_obj, value):
    tc_obj.delay_ms = value
    tc_obj._TrafficControl__validate_network_delay()


@pytest.mark.parametrize(["value", "expected"], [
    [-1, ValueError],
    [10001, ValueError],
])
def test_TrafficControl_validate_network_delay_exception(tc_obj, value, expected):
    tc_obj.delay_ms = value
    with pytest.raises(expected):
        tc_obj._TrafficControl__validate_network_delay()


@pytest.mark.parametrize(["value"], [
    [None],
    [0],
    [99],
])
def test_TrafficControl_validate_packet_loss_rate_normal(tc_obj, value):
    tc_obj.loss_percent = value
    tc_obj._TrafficControl__validate_packet_loss_rate()


@pytest.mark.parametrize(["value", "expected"], [
    [-0.1, ValueError],
    [99.1, ValueError],
])
def test_TrafficControl_validate_packet_loss_rate_exception(tc_obj, value, expected):
    tc_obj.loss_percent = value
    with pytest.raises(expected):
        tc_obj._TrafficControl__validate_packet_loss_rate()


@pytest.mark.parametrize(["value"], [
    [None],
    [""],

    ["192.168.0.1"],
    ["192.168.0.254"],

    ["192.168.0.1/32"],
    ["192.168.0.0/24"],
])
def test_TrafficControl_validate_network_normal(tc_obj, value):
    tc_obj.network = value
    tc_obj._TrafficControl__validate_network()


@pytest.mark.parametrize(["value", "expected"], [
    ["192.168.0.", ValueError],
    ["192.168.0.256", ValueError],

    ["192.168.0.0/0", ValueError],
    ["192.168.0.0/33", ValueError],
    ["192.168.0.2/24", ValueError],
    ["192.168.0.0000/24", ValueError],
])
def test_TrafficControl_validate_network_exception(tc_obj, value, expected):
    tc_obj.network = value
    with pytest.raises(expected):
        tc_obj._TrafficControl__validate_network()


@pytest.mark.parametrize(["value"], [
    [None],
    [0],
    [65535],
])
def test_TrafficControl_validate_port_normal(tc_obj, value):
    tc_obj.port = value
    tc_obj._TrafficControl__validate_port()


@pytest.mark.parametrize(["value", "expected"], [
    [-1, ValueError],
    [65536, ValueError],
])
def test_TrafficControl_validate_port_exception(tc_obj, value, expected):
    tc_obj.port = value
    with pytest.raises(expected):
        tc_obj._TrafficControl__validate_port()



# encoding: utf-8

'''
@author: Tsuyoshi Hombashi
'''

import dataproperty
import ipaddress
import six
import thutils


class TrafficControl(object):
    __MIN_LOSS_RATE = 0  # [%]
    __MAX_LOSS_RATE = 99  # [%]

    __MIN_DELAY_MS = 0  # [millisecond]
    __MAX_DELAY_MS = 10000  # [millisecond]

    __MIN_BUFFER_BYTE = 1600

    __MIN_PORT = 0
    __MAX_PORT = 65535

    def __init__(self, subproc_wrapper, device):
        self.__subproc_wrapper = subproc_wrapper
        self.__device = device

        self.rate = None
        self.delay_ms = None
        self.loss_percent = None
        self.network = None
        self.port = None

    def validate(self):
        if dataproperty.is_empty_string(self.__device):
            raise ValueError("device name is empty")

        self.__validate_network_delay()
        self.__validate_packet_loss_rate()
        self.network = self.__validate_network()
        self.__validate_port()

    def set_tc(self):
        self.__make_qdisc()
        self.__set_pre_network_filter()
        self.__set_delay_and_loss()
        self.__set_network_filter()
        self.__set_rate()

    def delete_tc(self):
        command = "tc qdisc del dev %s root" % (self.__device)

        return self.__subproc_wrapper.run(command)

    def __validate_network_delay(self):
        if self.delay_ms is None:
            return

        if self.delay_ms > self.__MAX_DELAY_MS:
            raise ValueError(
                "network delay is too high: expected<=%d[ms], value=%d[ms]" % (
                    self.__MAX_DELAY_MS, self.delay_ms))

        if self.delay_ms < self.__MIN_DELAY_MS:
            raise ValueError(
                "delay time is too low: expected>=%d[ms], value=%d[ms]" % (
                    self.__MIN_DELAY_MS, self.delay_ms))

    def __validate_packet_loss_rate(self):
        if self.loss_percent is None:
            return

        if self.loss_percent > self.__MAX_LOSS_RATE:
            raise ValueError(
                "packet loss rate is too high: expected<=%d[%%], value=%d[%%]" % (
                    self.__MAX_LOSS_RATE, self.loss_percent))

        if self.loss_percent < self.__MIN_LOSS_RATE:
            raise ValueError(
                "packet loss rate is too low: expected>=%d[%%], value=%d[%%]" % (
                    self.__MIN_LOSS_RATE, self.loss_percent))

    def __validate_network(self):
        if dataproperty.is_empty_string(self.network):
            return ""

        try:
            ipaddress.IPv4Address(six.u(self.network))
            return self.network + "/32"
        except ipaddress.AddressValueError:
            pass

        ipaddress.IPv4Network(six.u(self.network))
        return self.network

        raise ValueError("unrecognizable network: " + self.network)

    def __validate_port(self):
        if self.port is None:
            return

        if self.port > self.__MAX_PORT:
            raise ValueError(
                "port number is too high: expected<=%d[%%], value=%d[%%]" % (
                    self.__MAX_PORT, self.port))

        if self.port < self.__MIN_PORT:
            raise ValueError(
                "port number is too low: expected>=%d[%%], value=%d[%%]" % (
                    self.__MIN_PORT, self.port))

    def __make_qdisc(self):
        command_list = [
            "tc qdisc add",
            "dev " + self.__device,
            "root handle 1: prio",
        ]

        return self.__subproc_wrapper.run(" ".join(command_list))

    def __set_pre_network_filter(self):
        if dataproperty.is_empty_string(self.network):
            flowid = "1:1"
        else:
            flowid = "1:2"

        command_list = [
            "tc filter add",
            "dev " + self.__device,
            "protocol ip parent 1: prio 2 u32 match ip dst 0.0.0.0/0",
            "flowid " + flowid
        ]

        return self.__subproc_wrapper.run(" ".join(command_list))

    def __set_delay_and_loss(self):
        command_list = [
            "tc qdisc add",
            "dev " + self.__device,
            "parent 1:1 handle 10: netem",
        ]
        if self.loss_percent > 0:
            command_list.append("loss %s%%" % (self.loss_percent))
        if self.delay_ms > 0:
            command_list.append("delay %dms" % (self.delay_ms))

        return self.__subproc_wrapper.run(" ".join(command_list))

    def __set_network_filter(self):
        if all([
            dataproperty.is_empty_string(self.network),
            self.port is None,
        ]):
            return 0

        command_list = [
            "tc filter add",
            "dev " + self.__device,
            "protocol ip parent 1: prio 1 u32",
            "flowid 1:1",
        ]
        if dataproperty.is_not_empty_string(self.network):
            command_list.append("match ip dst " + self.network)
        if self.port is not None:
            command_list.append("match ip dport %d" % (self.port))

        return self.__subproc_wrapper.run(" ".join(command_list))

    def __set_rate(self):
        if dataproperty.is_empty_string(self.rate):
            return 0

        rate_kbps = thutils.common.humanreadable_to_byte(
            self.rate, kilo_size=1000) / 1000.0
        if rate_kbps <= 0:
            raise ValueError("rate must be greater than zero")

        command_list = [
            "tc qdisc add",
            "dev " + self.__device,
            "parent 10:1",
            "handle 20:",
            "tbf",
            "rate %dkbit" % (rate_kbps),
            "buffer %d" % (max(rate_kbps, self.__MIN_BUFFER_BYTE)),  # [byte]
            "limit 10000",
        ]

        return self.__subproc_wrapper.run(" ".join(command_list))
